/*
 * Mangle unix filenames to cybiko interpreter standards of
 * 10.3 format
**** All rights reserved.
****************************************************************************
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details at www.gnu.org
****************************************************************************
 */
#include "cyucon.h"
#include <ctype.h> /* isupper, tolower */
/*
Obtain basename of a file.
 */
const char *baseName(const char *inName)
{
  const char *retVal=strrchr(inName,'/');
  
  if(! retVal)
    retVal=inName;
  else
    ++retVal;
  return retVal;
}
/*
Remove characters from a sendfile name.
 */
int removeChars(const char *inName,char *outName,char *charsToRemove,
int removeAll)
{
  int status=STAT_NEUTRAL;
  char *removePtr=0;
  char *workBuffer=strdup(inName);
  
  removePtr=strpbrk(workBuffer,charsToRemove);

  if(removePtr)
  {
    strcpy(removePtr,removePtr+1);
    if(( strlen(workBuffer) <= MAX_SENDFILE_ROOTNAME) && (! removeAll))
       status=STAT_SUCCESS;
  }

  if(STAT_NEUTRAL == status)
  {
    while((removePtr) && (STAT_NEUTRAL == status) )
    {
      removePtr=strpbrk(removePtr,charsToRemove);
      if(removePtr)
	strcpy(removePtr,removePtr+1);
      if( (strlen(workBuffer) <= MAX_SENDFILE_ROOTNAME) && (! removeAll) )
	status=STAT_SUCCESS;
    }
  }
  strcpy(outName,workBuffer);
  free(workBuffer);
  return status;
}
/*
Edit input filename to maximum of 10 chars, one ., and a 3-char
extension.
 */
int fixFileNameIntoCybiko(const char *inFilename,char *outFilename)
{
  char *workPtr;
  char *badCharacter;
  char *workFileName = strdup(baseName(inFilename));
  char *workFileName2 = calloc(1,strlen(workFileName)+1);
  int status=STAT_NEUTRAL;
  char extensionHolder[5];

  memset(extensionHolder,0,sizeof(extensionHolder));
  // No spaces, tabs, or other mush.
  removeChars(workFileName,workFileName2," \t\n\r",1);
  strcpy(workFileName,workFileName2);

  // Ensure maximum of one period to a customer.
  workPtr=strrchr(workFileName,'.');
  if(workPtr)
  {
    *workPtr='\0';
    while(badCharacter)
    {
      badCharacter=strrchr(workFileName,'.');
      if(badCharacter)
	*badCharacter='_';
    }
    *workPtr='.';
    // Extension must be no more than 3 chars.
    while(strlen(workPtr) > (MAX_SENDFILE_EXTENSION+1) )
    {
      workPtr[strlen(workPtr)-1] = '\0';
    }
    strcpy(extensionHolder,workPtr);
    *workPtr=0;
  }
  
  if(MAX_SENDFILE_ROOTNAME >= strlen(workFileName))
     status=STAT_SUCCESS;

  if(STAT_NEUTRAL == status)
  {
    status=removeChars(workFileName,workFileName2,
"~`!@#$%^&*()_-+=?/><,\"':;[{}]",0);
    strcpy(workFileName,workFileName2);
  }
  if(STAT_NEUTRAL == status)
  {
    status=removeChars(workFileName,workFileName2,"AaEeIiOoUuYy",0);
    strcpy(workFileName,workFileName2);
  }
  if(STAT_NEUTRAL == status)
  {
    workPtr=workFileName + MAX_SENDFILE_ROOTNAME;
    *workPtr='\0';
  }
  if(*extensionHolder)
  {
    strcat(workFileName,extensionHolder);
  }
  /*  
  workPtr=workFileName;
  while(*workPtr)
  {
    if(isupper(*workPtr))
      *workPtr=tolower(*workPtr);
    ++workPtr;
  }
  */

  strcpy(outFilename,workFileName);
  
  free(workFileName2);
  free(workFileName);

  return STAT_SUCCESS;

}

/*
 *   Remove directory path from input filename.
 */
int fixFileNameOutOfCybiko(const char *inFilename,char *outFilename)
{
  strcpy(outFilename,baseName(inFilename));
  return STAT_SUCCESS;
}
/*
 * Prepend "/a/" to filename so we can talk to memory card.
 */
int fileNameToMemCard(const char *fileName,char *outBuf,size_t sizeOfOutBuf)
{
  int status=STAT_SUCCESS;
  
  if(sizeOfOutBuf < (strlen(fileName) + strlen(MEMCARDDIRPREP)))
     status=STAT_FAILURE;
  if(STAT_SUCCESS == status)
  {
    memset(outBuf,0,sizeOfOutBuf);
    strcpy(outBuf,MEMCARDDIRPREP);
    strcat(outBuf,fileName);
  }
  return status;
}
