/*
 * Send a single command to the Cybiko.
 * Jeff Frohwein. 
 
**** All rights reserved.
****************************************************************************
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details at www.gnu.org
****************************************************************************
 *
 */
#include "cyucon.h"
#include "crctab.h"
#include <stdio.h> /* bufsize, perror */

/*
Send a command, wait for prompt
 */
void SendCommand (int pf, char *command,size_t cmdLen, 
char *terminateIndicator, int echo, int watchStdIn)
{
    char term[4];
    int done = 0;
    size_t termIndicatorStrlen;
    size_t redBufSize;
    size_t totalCmdCharsRead=0;
    size_t commandSize=cmdLen;
    char *terminateTestPtr;
    int crc = crc1(command,cmdLen);
    fd_set  ready;        /* used for select */
    char buf[BUFSIZ];

    if(terminateIndicator)
      termIndicatorStrlen=strlen(terminateIndicator);
    if(! commandSize)
      commandSize=strlen(command);
    
    Rcv[0] = 0;
    term[0] = 0xa;
    term[1] = crc & 0xff;
    term[2] = crc >> 8;

    write(pf, command, commandSize);
    write(pf, term, 3);
    
    // wait until done
    while (!done)
    {
	FD_ZERO(&ready);
	FD_SET(STDIN_FILENO, &ready);
	FD_SET(pf, &ready);

	select(pf+1, &ready, NULL, NULL, NULL);

	// If data received, send it to screen
	if (FD_ISSET(pf, &ready))
	{
	    /* pf has characters for us */
	    redBufSize = read(pf, buf, BUFSIZ);
	    if (redBufSize)
            {
		buf[redBufSize]=0;
		strcat(Rcv, buf);
		totalCmdCharsRead += redBufSize;
		if(echo)
		    write(STDOUT_FILENO, buf, redBufSize);
            }
	}
	
	// Terminate if user presses any button
	if(watchStdIn) 
	{
	    if (FD_ISSET(STDIN_FILENO, &ready))
	    {
		/* standard input has characters for us */
		redBufSize = read(STDIN_FILENO, buf, BUFSIZ);
		if (redBufSize)
		    done = 1;
	    }
	}

	/* 
	   Terminate upon completion of data, 
	   assuming no comment from stdin. 
	*/
	if((! done) && (terminateIndicator)) {
	    terminateTestPtr=( Rcv + totalCmdCharsRead ) - termIndicatorStrlen;
	    if(0 == strcmp(terminateTestPtr,terminateIndicator))
		done=1;
	}
	else {
	  done=1; /* Null terminate indicator means return immediately. */
	}
    }
	return;
}
