/* 
Send a file to the cybiko. Jeff Frohwein. 

**** All rights reserved.
****************************************************************************
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details at www.gnu.org
****************************************************************************
*/

#include "cyucon.h"
#include "crctab.h"

#include <stdio.h> /* bufsize, perror */
#include <sys/stat.h> /* struct stat */
#include <sys/types.h> /* select */
#include <unistd.h> /* select */


/*
 * Calculate a binary buffer header ("C, sequence, 0, buffer size)
 */
static void calcBinHeader(const char *buffer, unsigned int bufsize,
unsigned int sequenceNum, char *headerBuf)
{
    *headerBuf='C';
    headerBuf[1]=sequenceNum & 0x00ff;
    headerBuf[2]=sequenceNum >> 8;
    headerBuf[3]=bufsize & 0xff;
    headerBuf[4]=bufsize >> 8;
       
    return;
}
/*
 * Calculate a binary buffer trailer (CRC and sequence)
 */
static void calcBinTerminator(const char *sendBuffer,unsigned int bufferSize,
unsigned int seqno,char *outbuf)
{
    unsigned int eax,ebx,ecx,esi,crc2;
    int ii;

    eax = seqno;
    ebx = bufferSize;
    
    esi = CRCTable[(eax ^ 0xff) & 0xff] ^ 0xffffff;
    eax = CRCTable[(esi & 0xff) ^ ((seqno >> 8) & 0xff)];
    esi = esi >> 8;
    eax = eax ^ esi;
    ecx = CRCTable[(eax & 0xff) ^ (ebx & 0xff)] ^ (eax >> 8);
    eax = CRCTable[((ebx >> 8) & 0xff) ^ (ecx & 0xff)] ^ (ecx >> 8);
    
    for (ii=0; ii<bufferSize; ii++)
	eax = (eax >> 8) ^  CRCTable[(eax & 0xff) ^ (sendBuffer[ii] & 0xff)];
    
    crc2 = eax ^ 0xffffffff;
    
    *outbuf = crc2 & 0xff;
    outbuf[1] = (crc2 >> 8) & 0xff;
    outbuf[2] = (crc2 >> 16) & 0xff;
    outbuf[3] = (crc2 >> 24) & 0xff;
    MaybePrintMessage(DEBUG,"CRC2 is: %hX %hX %hX %hX\n",
    outbuf[0],outbuf[1],outbuf[2],outbuf[3]); 

    return;
}

/*
 Send a file to the Cybiko.
 */
void SendFile (int pf, char *fileName)
{
    FILE *fptr;
    fpos_t pos;
    char header[5];
    char terminator[4];
    char size[8];
    char tmpbuf[80];
    char rcvstr[] = {"rcv "};
    int done = 0;
    int i;
    unsigned int packetSequence;
    unsigned int numread;
    fd_set ready;        /* used for select */
    static char buf[BUFSIZ];
    unsigned char fbuf[SENDPACKETDATASIZE+1];
    char *workPtr;
    struct stat inFileStat;
    char *sendFileName=calloc(1,strlen(fileName)+5);
    
    if(MangleFilename)
      fixFileNameIntoCybiko(fileName,sendFileName);
    else
      strcpy(sendFileName,fileName);
    if(MemoryFlash) 
    {
      char *temporaryBuf=calloc(1,strlen(sendFileName)+5);
      fileNameToMemCard(sendFileName,temporaryBuf,strlen(sendFileName)+5);
      strcpy(sendFileName,temporaryBuf);
      free(temporaryBuf);
    }
    if(-1 == stat(fileName,&inFileStat))
    {
      MaybePrintMessage(ERROR,"File %s: %s",fileName,strerror(errno));
      return;
    }
    sprintf(size, " %ld", (inFileStat.st_size)-1);
    fptr = fopen(fileName, "r");
    
    strcpy(tmpbuf,rcvstr);
    strcat(tmpbuf,sendFileName);
    strcat(tmpbuf,size);
    
    i = crc1(tmpbuf,0);
    memset(terminator,0,sizeof(terminator));

    terminator[0] = 0xa;
    terminator[1] = i & 0xff;
    terminator[2] = i >> 8;
    write(pf, tmpbuf, strlen(tmpbuf));
    write(pf, terminator, 3);

    *Rcv = 0; 

    // wait until done
    while (!done)
    {
	FD_ZERO(&ready);
	FD_SET(STDIN_FILENO, &ready);
	FD_SET(pf, &ready);

        select(pf+1, &ready, NULL, NULL, NULL);
	
	// If data received, send it to screen
	if (FD_ISSET(pf, &ready))
	{
	    /* pf has characters for us */
	    i = read(pf, buf, sizeof(buf)-1);
	    if (i > 0)
            {
		buf[i]=0;
		strcat(Rcv, buf);
		write(STDOUT_FILENO, buf, i);	    
            }
	}
	
	// Terminate if user presses any button
	if (FD_ISSET(STDIN_FILENO, &ready))
	{
	    /* standard input has characters for us */
	    i = read(STDIN_FILENO, buf, BUFSIZ);
	    if (i > 0)
		done = 1;
	}
	
	// Send next packet
	if ( (Rcv[strlen(Rcv)-1]==0x0a) &&
	     (Rcv[strlen(Rcv)-2]==0x0d) )
        {
	    workPtr=rindex(Rcv,'d');
	    sscanf(workPtr,"d%d",&packetSequence);
	    MaybePrintMessage(PROGRESS,"[Send packet %d]",packetSequence);
	    
	    /* Set file position to start of packet. Do this */
	    /* in case we missed a packet and are re-sending. */
	    pos = packetSequence << 9;
	    fsetpos(fptr, &pos);
	    
	    // Read a packet from disk
	    numread = fread(fbuf, 1, SENDPACKETDATASIZE, fptr);

	    calcBinHeader(fbuf,numread,packetSequence,header);
	    write(pf, header, 5);
	    write(pf,fbuf,numread); 
	    calcBinTerminator(fbuf,numread,packetSequence,terminator); 
	    write(pf, terminator, 4); 
        }
	
	// Terminate upon completion of data
      if ( (Rcv[strlen(Rcv)-1]==0x20) &&
           (Rcv[strlen(Rcv)-2]==0x0a) &&
           (Rcv[strlen(Rcv)-3]==0x0d) &&
           (Rcv[strlen(Rcv)-4]=='>') &&
           (Rcv[strlen(Rcv)-5]==' ') &&
           (Rcv[strlen(Rcv)-6]=='y') &&
           (Rcv[strlen(Rcv)-7]=='d') &&
           (Rcv[strlen(Rcv)-8]=='a') &&
           (Rcv[strlen(Rcv)-9]=='e') &&
           (Rcv[strlen(Rcv)-10]=='R') )
        done = 1;
    }    
    free(sendFileName);
    fclose(fptr);
}
